Imports System

Namespace SpikingNeuron
    Friend Class SpikingNeuronProgram
        Shared Sub Main(ByVal args() As String)
            Console.WriteLine(vbCrLf & "Begin spiking neuron demo" & vbCrLf)

            Dim inputs(2)() As Integer
            inputs(0) = New Integer() {0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1}
            inputs(1) = New Integer() {1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1}
            inputs(2) = New Integer() {1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1}
            ' t =                  0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15

            Console.WriteLine("The inputs are: " & vbCrLf)
            For i = 0 To inputs.Length - 1
                ShowVector(inputs(i), False)
            Next i
            Console.WriteLine("")

            Dim output(15) As Integer ' process 16 input values

            Dim wts() As Integer = {4, -2, 3}
            Console.Write("The weights are: ")
            ShowVector(wts, True)
            Console.WriteLine("")

            Dim leak As Integer = 1
            Console.WriteLine("The leak potential is:           " & leak)

            Dim v As Integer = 0 ' electrical potential (voltage)
            Dim thresh As Integer = 8 ' potential value needed to fire an output spike
            Dim spike As Integer = 4 ' increase in v at spike event
            Dim tNext As Integer = 0 ' next time when neuron is active
            Dim latency As Integer = 2 ' number t inactive after spike

            Console.WriteLine("The threshold potential is:      " & thresh)
            Console.WriteLine("The spike potential is:          " & spike)
            Console.WriteLine("The post-spike latency time is:  " & latency)
            Console.WriteLine(vbCrLf & "Starting processing" & vbCrLf)

            For t = 0 To 15
                Console.WriteLine("---------------------------------------------------")
                Console.Write(" ")
                Console.Write("t = ")
                If t <= 9 Then
                    Console.Write(" ")
                End If
                Console.Write(t & ". ")

                Console.Write("Inputs = " & inputs(0)(t) & " " & inputs(1)(t) & " " & inputs(2)(t))

                If t <> tNext Then ' neuron is not active
                    Console.Write(". Neuron is inactive. ")
                    Console.WriteLine("V = " & v)
                    output(t) = 0
                Else ' neuron is active
                    Console.Write(". Neuron is   active. ")

                    Dim sum As Integer = 0
                    For j = 0 To inputs.Length - 1
                        sum += inputs(j)(t) * wts(j)
                    Next j
                    v = v + sum

                    v = v - leak
                    If v < 0 Then
                        v = 0
                    End If

                    Console.WriteLine("V = " & v)

                    If v >= thresh Then ' spike and reset
                        v = v + spike
                        Console.WriteLine(" Spiking, V = " & v)
                        output(t) = 1
                        v = 0
                        tNext = t + 1 + latency
                    Else
                        output(t) = 0
                        tNext = t + 1
                    End If
                End If ' active
            Next t ' t
            Console.WriteLine("---------------------------------------------------" & vbCrLf)
            Console.WriteLine("Output spike train = ")
            ShowVector(output, False)

            Console.WriteLine(vbCrLf & "End spiking neuron demo" & vbCrLf)
            Console.ReadLine()
        End Sub ' Main

        Private Shared Sub ShowVector(ByVal vector() As Integer, ByVal plus As Boolean)
            For i = 0 To vector.Length - 1
                If plus = True AndAlso vector(i) >= 0 Then
                    Console.Write("+")
                End If
                Console.Write(vector(i) & " ")
            Next i
            Console.WriteLine("")
        End Sub
    End Class ' Program
End Namespace ' ns
